use warnings;
use strict;

package Net::Domain::SMD;
use base 'Exporter';

our @EXPORT_OK   = qw/SMD10_NS MARK10_NS/;
our %EXPORT_TAGS =
  ( ns10 => [ qw/SMD10_NS MARK10_NS/ ]
  );

use Log::Report                  'net-domain-smd';
use XML::Compile::Cache          ();
use XML::Compile::WSS::Signature ();
use XML::Compile::WSS::Util      qw(DSIG_NS);
use Net::Domain::SMD::File       ();

use constant
  { SMD10_NS  => 'urn:ietf:params:xml:ns:signedMark-1.0'
  , MARK10_NS => 'urn:ietf:params:xml:ns:mark-1.0'
  };

my %prefixes =
  ( ds   => DSIG_NS   # do not take this prefix from these schemas
  , smd  => SMD10_NS
  , mark => MARK10_NS
  );

=chapter NAME
Net::Domain::SMD - Mark and Signed-Mark Objects Mapping

=chapter SYNOPSIS

  # This object is used by Net::Domain::TMCH, so usually not directly
  # accessed by end-users.

  my $smd  = Net::Domain::SMD->new(%options);
  my $info = $smd->read($fn);

=chapter DESCRIPTION
Draft RFC L<http://tools.ietf.org/html/draft-lozano-tmch-smd-02>
specifies a way for TLD (domain registries) to exchange mark
rights (brand registrations): "SMD"-files.

This module unpacks these SMD-files, decoding the XML contained in them.
Each SMD-file is represented by a separate M<Net::Domain::SMD::File>
object.

=chapter METHODS

=section Constructors

=c_method new OPTIONS

=option  prepare 'NONE'|'READER'
=default prepare 'READER'
=cut

sub new($%) { my ($class, %args) = @_; (bless {}, $class)->init(\%args) }
sub init($)
{   my ($self, $args) = @_;

    (my $xsddir = __FILE__) =~ s!\.pm!/xsd/!;
    my @xsds    =
      ( "$xsddir/mark-1.0.xsd"
      , "$xsddir/mark-1.0-bugs.xsd"
      , "$xsddir/signedMark-1.0.xsd"
      , "$xsddir/signedMark-1.0-bugs.xsd"
      );

    my $schemas = $self->{NDS_schemas}
      = XML::Compile::Cache->new(\@xsds, prefixes => \%prefixes);

    # do not prefix 'mark', because the accesses it all the time.
    $schemas->addKeyRewrite('PREFIXED(smd)');
    my $sig = XML::Compile::WSS::Signature->new
      ( schema     => $schemas
      , prepare    => ($args->{prepare} || 'READER')
      , sign_types => [ 'smd:signedMarkType', 'ds:KeyInfoType' ]
      , sign_put   => 'smd:signedMarkType'
      , sign_when  => 'smd:signedMarkType'
      );
    $self;
}

#-------------------------

=section Attributes

=method schemas
=cut

sub schemas()     {shift->{NDS_schemas}}

#-------------------------

=section Actions

=method read FILENAME
Returns an M<Net::Domain::SMD::File> object, or an exception which can
be caught with M<Log::Report::try()> or C<eval>.
=cut

sub read($)
{   my ($self, $fn) = @_;
    Net::Domain::SMD::File->fromFile($fn, schemas => $self->schemas);
}

1;
