use warnings;
use strict;

package Net::Domain::SMD::RL;
use base 'Exporter';

use Log::Report                  'net-domain-smd';
use Scalar::Util        qw(blessed);

=chapter NAME
Net::Domain::SMD::RL - SMD Revokation List

=chapter SYNOPSIS

  my $crl  = Net::Domain::SMD::RL->fromFile($fn, %options);
  my $crl  = Net::Domain::SMD::RL->fromURI($uri, %options);

  if($rl->isRevoked($smd)) ...
  if($rl->isRevoked($smd)) ...

  my $tmch = Net::Domain::TMCH->new
    ( ...
    , smd_revocations => ...
    );
  my @rl = $tmch->smdRevocations;

=chapter DESCRIPTION
Draft RFC L<http://tools.ietf.org/html/draft-lozano-tmch-func-07>
specifies a way for TLD (domain registries) to exchange mark
rights (brand registrations).  This module manages the files which
list revocation of SMD files.

=chapter METHODS

=section Constructors

=c_method new OPTIONS

=requires source STRING|FILENAME|URI
Identification for the source of this list, to be used in error message
and such.

=option  revoked HASH|ARRAY
=default revoked []
ARRAY of serial numbers from revoked certificates, or a HASH of which the
keys are the revoked serials.
=cut


sub new($%) { my ($class, %args) = @_; (bless {}, $class)->init(\%args) }

sub init($)
{   my ($self, $args) = @_;
    $self->{NDSR_source}  = $args->{source} or panic;

    my $rev =  $args->{revoked} || [];
    $rev = { map +($_ => 1), @$rev} if ref $rev eq 'ARRAY';
    $self->{NDSR_revoked} = $rev;
    $self;
}

=method fromFile FILENAME, OPTIONS
Read a RL from file, for instance a cached or test RL.
The OPTIONS are passed to M<new()>.
=cut

sub _process(@)
{   my $self = shift;
    my $revoked = $self->{NDSR_revoked} ||= {};

    # Compact code: needs to be fast.
    # be warned: \n may end line, but last element not used (yet).
    my ($version, $timestamp) = shift;
    my $header  = shift;
    $revoked->{lc +(split /\,/, $_, 2)[0]} = 1 for @_;
    $self;
}

sub fromFile($%)
{   my ($class, $fn) = (shift, shift);

    open my($fh), '<:raw', $fn
        or fault "cannot read RL file {fn}", fn => $fn;

    my $self = $class->new(source => $fn, @_);
    $self->_process($fh->getlines);
}

=method fromURI URI, OPTIONS
Create a new RL object from an URI, which is collected dynamically.  The
OPTIONS are passed to M<new()>.
=cut

my $ua;
sub fromURI($%)
{   my ($class, $uri) = (shift, shift);

    eval "require LWP::UserAgent";
    $@ and error __x"need LWP::UserAgent to fetch RL: {err}", err => $@;

    $ua ||= LWP::UserAgent->new;
    my $resp = $ua->get($uri);
    $resp->is_success
        or error __x"could not collect RL from {source}: {err}"
              , $resp->status_line;

    my $self = $class->new(source => $uri, @_);
    $self->_process(split /\r?\n/, $resp->decoded_content);
}

#-------------------------

=section Attributes
=method source
=cut

sub source() {shift->{NDSR_source}}

#-------------------------

=section Actions

=method isRevoked SMD|SMDID
=cut

sub isRevoked($)
{   my ($self, $smd) = @_;
    my $smdid = blessed $smd ? $smd->smdID : $smd;
    exists $self->{NDSR_revoked}{lc $smdid};
}

1;
